/***********************************************************
*     a FREE MMC/SD-interface to SPI-slot of GD32VF103C    *
*                                                          *
*          by  Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                      FFSPort_MMC.c                       *
*                   MMC/SD TO SPI PORT                     *
************************************************************
*      - code generation            Frank Goetze   11/2019 *
***********************************************************/

#include "../../../inc/MMC/GD32VF103C/FFSPort_SPI.h"

/****************** constant variables ********************/
#define CPU_CRISTAL_CLK  8000000UL                         // processor crystal-clock in Hz
OS_CONST U16 OS_ROMDATA SPI_c[] = {
    SPI_PSC_2,
    SPI_PSC_4,
    SPI_PSC_8,
    SPI_PSC_16,
    SPI_PSC_32,
    SPI_PSC_64,
    SPI_PSC_128,
    SPI_PSC_256
};

/*
************************************************************
*          set max possible baudrate and return this
************************************************************
*/
U32 FFSPort_MMC_SetBR(U32 maxclk)
{
    U32                s_clksrc;
    U08                i;
    
    i = 0;
    s_clksrc = rcu_clock_freq_get(CK_APB1);   // SPI1
    while(((s_clksrc / (2UL << i)) > maxclk) && (i < 7)) i++;
    SPI_CTL0(SPIx) = (SPI_CTL0(SPIx) & ~SPI_PSC_256) | SPI_c[i];
    return(s_clksrc / (2UL << i));
}

#ifdef USE_MMC_DMA
OS_CONST U08 dbFF = 0xFF;
/*
************************************************************
*            transfer a block via DMA to device
************************************************************
*/
U08 FFSPort_MMC_TxBlock(U08 OS_HUGE *block, U32 len)
{
    return(0);
}

/*
************************************************************
*            receive a block via DMA from device
************************************************************
*/
U08 FFSPort_MMC_RxBlock(U08 OS_HUGE *block, U32 len)
{
    return(0);
}
#endif
  
/*
************************************************************
*            write a char and read one back
************************************************************
*/
U08 FFSPort_MMC_Send(U08 w)
{
    while (spi_i2s_flag_get(SPIx, SPI_FLAG_TBE) == RESET) _NOP();   // wait up to tx-empty    // or SPI_FLAG_TRANS
    spi_i2s_data_transmit(SPIx, (U16)w);                            // write char and send
    while (spi_i2s_flag_get(SPIx, SPI_FLAG_RBNE) == RESET) _NOP();  // wait up to rx-not_empty (char received)
    return (U08)spi_i2s_data_receive(SPIx);                         // return received char
}

/*
************************************************************
*              reinitialise the SPI-port
************************************************************
*/
U08 FFSPort_MMC_ReInit(void)
{
    FFSPort_MMC_SetBR(400000);                             // set SPI-clk to max 400kHz as startup (should never be > 400kHz, says MMC)
    return(0);                                             // return actual alltimes ok.
}

/*
************************************************************
*              initialise the SPI-port
************************************************************
*/
U08 FFSPort_MMC_Init(void)
{
    spi_parameter_struct  SPI_InitStruct;

    /* GPIOB Periph clock enable */
    rcu_periph_clock_enable(RCU_GPIOB);

    /* Configure SPI1 pins */
    gpio_init(SPIx_Gx, GPIO_MODE_AF_PP, GPIO_OSPEED_50MHZ, SPIx_bit);
    gpio_init(SPIx_Gx, GPIO_MODE_IN_FLOATING, GPIO_OSPEED_50MHZ, GPIO_PIN_14);

    /* Configure PB12 pin: out pushpull mode */
    gpio_init(CS_Gx, GPIO_MODE_OUT_PP, GPIO_OSPEED_50MHZ, CS_bit);

    /* Configure SPI1 */
    rcu_periph_clock_enable(RCU_SPI1);

    spi_i2s_deinit(SPIx);
    spi_struct_para_init(&SPI_InitStruct);
    SPI_InitStruct.device_mode           = SPI_MASTER;
    SPI_InitStruct.trans_mode            = SPI_TRANSMODE_FULLDUPLEX;
    SPI_InitStruct.frame_size            = SPI_FRAMESIZE_8BIT;
    SPI_InitStruct.nss                   = SPI_NSS_SOFT;
    SPI_InitStruct.endian                = SPI_ENDIAN_MSB;
    SPI_InitStruct.clock_polarity_phase  = SPI_CK_PL_HIGH_PH_2EDGE;
    SPI_InitStruct.prescale              = SPI_PSC_256;
    spi_init(SPIx, &SPI_InitStruct);
    spi_enable(SPIx);

    return(FFSPort_MMC_ReInit());                          // (re)init SPI-clock
}

/************************** END ***************************/
